#!/usr/bin/perl

use strict;
use warnings;

# incoming
my %param = @ARGV;
my $prog = $param{-program};

# pre-process patterns
my %pre_pattern = (
						$prog . ".c" => "gcc " . $prog . ".c -o " . $prog . "_c",
						$prog . ".cc" => "g++ " . $prog . ".cc -o " . $prog . "_cc",
						$prog . ".java" => "javac " . $prog . ".java",
					);
					
# runtime patterns
my %run_pattern = (
						$prog . "_c" => "./" . $prog . "_c",
						$prog . "_cc" => "./" . $prog . "_cc",
						$prog . ".class" => "java " . $prog,
						$prog . ".pl" => "perl " . $prog . ".pl",
						$prog . ".py" => "python " . $prog . ".py",
						$prog . ".rb" => "ruby " . $prog . ".rb",
					);
					
# run the pre processor
foreach my $pre_file (keys %pre_pattern) {

	if (-f $pre_file) {
		print "Compiling $pre_file\n";
		my $res = `$pre_pattern{$pre_file}`;
	}
}

# setup our data structure
my %letter;
my $source = $param{-text};

foreach my $run_file (keys %run_pattern) {

	print "Running $run_file\n";
	my $results = `$run_pattern{$run_file} $source`;
	
	my @lines = split('\n', $results);
	foreach my $line (@lines) {
	
		if ($line =~ m/^([a-z]):\s+(.*)$/) {
		
			my $let = $1;
			my $freq = $2;
			if (! defined $letter{$let}) {
				$letter{$let} = {};
			}
			
			$letter{$let}{$run_file} = $freq;
		}
	}
}

# print our results
foreach my $let (sort keys %letter) {

	print "$let\n";
	my $last;
	foreach my $run (sort keys %{$letter{$let}}) {
		
		if (! defined $last) {
			$last = $letter{$let}{$run};
		}
		else {
			if ($last ne $letter{$let}{$run}) {
				print "\t\tCalculated value for $run deviates from previous values\n";
			}
			$last = $letter{$let}{$run};
			
		}
		
		print "\t" . $letter{$let}{$run} . " " . $run . "\n";
	}
}